import java.io.BufferedReader;
import java.io.FileReader;
import java.io.PrintWriter;
import java.util.ArrayList;

public class Requirements {
	String id;
	Double change_frequency;
	Double internal_probability;
	ClassList implementedIn;
	ClassList coChangeWith; // This is the list of requirements that co-changed at least ones
	
	public Requirements(String id_) {
		id = id_;
		change_frequency = 0.0;
		implementedIn = new ClassList();
		coChangeWith = new ClassList();
	}

	public boolean isCurrent(String creq) {
		return id.equals(creq);
	}

	public void addClasses(ArrayList<String> curr_classes) {
		for (int i=0;i<curr_classes.size();i++) {
			if (implementedIn.containsClass(curr_classes.get(i))) {
				implementedIn.increaseFrequency(curr_classes.get(i));
			} else {
				Clazz c = new Clazz(curr_classes.get(i));
				implementedIn.addClass(c);
			}
		}
	}

	public void writeFile(PrintWriter writer) {
		writer.println(id + "; " + change_frequency + "; " + internal_probability + "; " + implementedIn.getAsString() + "; " + coChangeWith.getAsString());		
	}

	public void increaseFrequency() {
		change_frequency++;
	}

	public void updateInternal(int total_commits) {
		internal_probability = change_frequency / total_commits;
		implementedIn.updateInternal(change_frequency);
		coChangeWith.updateInternal(change_frequency);
	}

	public void addCoChange(String rname) {
		if (coChangeWith.containsClass(rname)) {
			coChangeWith.increaseFrequency(rname);
		} else {
			Clazz c = new Clazz(rname);
			coChangeWith.addClass(c);
		}		
	}

	public Double getInternal() {
		return internal_probability;
	}

	public Double getCoChange(String fromRequirement) {
		Double rem = 0.0;
		for (int k=0;k<coChangeWith.size();k++) {
			if (coChangeWith.get(k).isCurrent(fromRequirement)) {
				rem = coChangeWith.get(k).getCoChange();
			}
		}
		return rem;
	}

	public Double getOverlappingContractsProbability(Requirements fReq, Double internal) {
		Double b2 = 0.0;
		
		for (int i=0; i<implementedIn.size();i++) {
			Clazz curr_class_to = implementedIn.get(i);
			if (fReq.isImplementedIn(curr_class_to)) {
				Double rem = fReq.getREM(curr_class_to);
				Double temp = rem * internal;
				b2 = b2 + temp - temp*b2;
			}
		}
		
		return b2;
	}

	private Double getREM(Clazz curr_class_to) {
		Double rem = 0.0;
		
		for (int i=0;i<implementedIn.size();i++) {
			if (implementedIn.get(i).getName().equals(curr_class_to.getName())) {
				rem = implementedIn.get(i).getCoChange();
				break;
			}
		}
		
		return rem;
	}

	private boolean isImplementedIn(Clazz curr_class_to) {
		boolean found = false;
		
		// We identify if the from requirement contains the "to" class
		for (int i=0;i<implementedIn.size();i++) {
			if (implementedIn.get(i).getName().equals(curr_class_to.getName())) {
				found = true;
				break;
			}
		}
		
		return found;
	}

	public Double getDirectRippleEffects(Requirements fReq, String fname, Double r_internal) {
		Double b31 = 0.0;
		
		for (int i=0; i<implementedIn.size();i++) {
			Clazz curr_class_to = implementedIn.get(i);
			for (int j=0; j<fReq.implementedIn.size(); j++) {
				Clazz curr_class_from = fReq.implementedIn.get(j);
				// Search in the REM file for the combination of "from" and "to" classes
				if (curr_class_from.getName().equals(curr_class_to.getName()) == false) {
					try (BufferedReader br = new BufferedReader(new FileReader(fname))) {
						String line;
						while ((line = br.readLine()) != null) {
					       String temp[] = line.split(";");
					       String from = temp[0].replaceAll("\\.", "/");
					       String to = temp[1].replaceAll("\\.", "/");;
					       Double rem = Double.parseDouble(temp[2]);					       
					       if (curr_class_from.getName().contains(from) && curr_class_to.getName().contains(to)) {
								Double class_internal = r_internal * fReq.getREM(curr_class_from); 
								Double temp_ripple = rem * class_internal;
								b31 = b31 + temp_ripple - temp_ripple * b31; 
								break;
					       }
						}
					} catch (Exception e){
						System.out.println(e.getMessage());
					}	
				}
			}
		}
		
		return b31;
	}

	public Double getIndirectRippleEffects(Requirements fReq, String fname, Double r_internal) {
		Double b32 = 0.0;
		
		for (int i=0; i<implementedIn.size();i++) {
			Clazz curr_class_to = implementedIn.get(i);
			for (int j=0; j<fReq.implementedIn.size(); j++) {
				Clazz curr_class_from = fReq.implementedIn.get(j);
				// Search in the REM file for the combination of "from" and "to" classes
				if (curr_class_from.getName().equals(curr_class_to.getName()) == false) {
					try (BufferedReader br = new BufferedReader(new FileReader(fname))) {
						String line;
						boolean found = false;
						while ((line = br.readLine()) != null) {
					       String temp[] = line.split(";");
					       String from = temp[0].replaceAll("\\.", "/");
					       String to = temp[1].replaceAll("\\.", "/");					       
					       if (curr_class_from.getName().contains(from) && curr_class_to.getName().contains(to)) {
					    	    found = true;
								break;
					       }
						}
						if (found==false) {
							try (BufferedReader br_new  = new BufferedReader(new FileReader(fname))) {
								while ((line = br_new.readLine()) != null) {
							       String temp[] = line.split(";");
							       String from = temp[0].replaceAll("\\.", "/");
							       String to = temp[1].replaceAll("\\.", "/");
							       Double rem = Double.parseDouble(temp[2]);
							       if (curr_class_to.getName().contains(to)) {
							    	   try (BufferedReader br_new_new  = new BufferedReader(new FileReader(fname))) {
											while ((line = br_new_new.readLine()) != null) {
										       String temp_new[] = line.split(";");
										       String from_new = temp_new[0].replaceAll("\\.", "/");
										       String to_new = temp_new[1].replaceAll("\\.", "/");
										       Double rem_new = Double.parseDouble(temp[2]);
										       if (from.contains(to_new) && curr_class_from.getName().contains(from_new)) {										    	   
													Double class_internal = r_internal * fReq.getREM(curr_class_from); 
													Double temp_ripple = rem * rem_new * class_internal;
													b32 = b32 + temp_ripple - temp_ripple * b32; 
													break;
										       }
											}
										} catch (Exception e){
											System.out.println(e.getMessage());
										}							    	   
							    	   break;
							       }
								}
							} catch (Exception e){
								System.out.println(e.getMessage());
							}	
						}
					} catch (Exception e){
						System.out.println(e.getMessage());
					}	
				}
			}
		}
		
		return b32;

	}

	public String getID() {
		return id;
	}
	
}
