import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;

import org.apache.commons.io.IOUtils;

public class RequirementsList {
	ArrayList<Requirements> rList = new ArrayList<Requirements>();
	int total_commits;
	String code;
	
	public RequirementsList(String code2) {
		code = code2;
	}
	
	public int size() {
		return rList.size();
	}

	public Requirements get(int i) {
		return rList.get(i);
	}
	
	public void populateRequirements(String fname) {
		try (BufferedReader br = new BufferedReader(new FileReader(fname))) {
		    String line;
		    while ((line = br.readLine()) != null) {
			       String temp[] = line.split(";");
			       Requirements req = new Requirements(temp[0]);
			       rList.add(req);
		    }
		} catch (Exception e){
			System.out.println(e.getMessage());
		}
	}

	public void calculateClassDependencies(String fname) throws IOException {
			
		total_commits = 0;
		
		try {
			FileInputStream fisTargetFile = new FileInputStream(new File(fname));
			String complete_file = IOUtils.toString(fisTargetFile, "UTF-8");		
			String commit_str[] = complete_file.split("commit");			       
			
			for (int i=1; i<commit_str.length; i++) {
				total_commits++;
				ArrayList<String> curr_req = new ArrayList<String>();
				ArrayList<String> curr_classes = new ArrayList<String>();
				
				String lines[] = commit_str[i].split("\n");
				for (int j=0; j<lines.length;j++) {
					if (lines[j].contains(code)) {
						//System.out.print("Requirement Found: ");
						String tokens[] = lines[j].split(" ");
						for (int k=0; k<tokens.length;k++) {
							if (tokens[k].contains(code)) {
								String creq = tokens[k]; 
								creq = creq.replace(":", "").replace("(", "").replace(")", "").replace(",", "").trim();
								curr_req.add(creq);
								//System.out.println(creq);
							}
						}
						
					}
					if (lines[j].endsWith(".java")) {
						curr_classes.add(lines[j]);
						//System.out.println("\t" + lines[j]);
					}
				}
				
				for (int j=0; j<curr_req.size();j++) {
					String creq = curr_req.get(j);
					//boolean found = false;
					for (int k=0;k<rList.size();k++) {
						if (rList.get(k).isCurrent(creq)) {
							rList.get(k).increaseFrequency();
							rList.get(k).addClasses(curr_classes);
							//found = true;
							break;
						}
					}
				}
				
				if (curr_req.size() > 1) {
					for (int j=0; j<curr_req.size();j++) {
						String creq = curr_req.get(j);
						for (int k=0;k<rList.size();k++) {
							if (rList.get(k).isCurrent(creq)) {
								for (int m=0; m<curr_req.size();m++) {
									if (curr_req.get(m).equals(creq)==false) {
										rList.get(k).addCoChange(curr_req.get(m));
									}
								}
								break;
							}							
						}
					}
				}
			}
			
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
	}

	public void writeFile() {
		try {
		    PrintWriter writer = new PrintWriter("output.csv", "UTF-8");
		    for (int i=0;i<rList.size();i++) {
				rList.get(i).updateInternal(total_commits);
		    	rList.get(i).writeFile(writer);
		    }
		    writer.close();
		} catch (IOException e) { }
	}

	public Double getInternalProbability(String toRequirement) {
		Double internal = 0.0;
		for (int k=0;k<rList.size();k++) {
			if (rList.get(k).isCurrent(toRequirement)) {
				internal = rList.get(k).getInternal();
			}
		}
		return internal;
	}

	public Double getCoChangeProbability(String fromRequirement, String toRequirement) {
		Double cochange = 0.0, internal = 0.0, rem = 0.0;

		for (int k=0;k<rList.size();k++) {
			if (rList.get(k).isCurrent(fromRequirement)) {
				internal = rList.get(k).getInternal();
			}
		}
		
		for (int k=0;k<rList.size();k++) {
			if (rList.get(k).isCurrent(toRequirement)) {
				rem = rList.get(k).getCoChange(fromRequirement);
			}
		}
		
		cochange = internal * rem;
		return cochange;
	}

	public Double getOverlappingContractsProbability(String fromRequirement, String toRequirement) {
		Double c_overlap = 0.0, internal = 0.0;

		for (int k=0;k<rList.size();k++) {
			if (rList.get(k).isCurrent(fromRequirement)) {
				internal = rList.get(k).getInternal();
			}
		}
		
		for (int k=0;k<rList.size();k++) {
			if (rList.get(k).isCurrent(toRequirement)) {
				for (int m=0;m<rList.size();m++) {
					if (rList.get(m).isCurrent(fromRequirement)) {
						Requirements fReq = rList.get(m); 
						c_overlap = rList.get(k).getOverlappingContractsProbability(fReq, internal);
					}
				}
			}
		}
		
		return c_overlap;
	}

	public Double getDirectRippleEffects(String fromRequirement, String toRequirement, String fname) {
		Double ripple_direct = 0.0, internal =0.0;
		
		for (int k=0;k<rList.size();k++) {
			if (rList.get(k).isCurrent(fromRequirement)) {
				internal = rList.get(k).getInternal();
			}
		}		
		
		for (int k=0;k<rList.size();k++) {
			if (rList.get(k).isCurrent(toRequirement)) {
				for (int m=0;m<rList.size();m++) {
					if (rList.get(m).isCurrent(fromRequirement)) {
						Requirements fReq = rList.get(m); 
						ripple_direct = rList.get(k).getDirectRippleEffects(fReq, fname, internal);
					}
				}
			}
		}
		
		return ripple_direct;			
	}

	public Double getIndirectRippleEffects(String fromRequirement, String toRequirement, String fname) {
		Double ripple_indirect = 0.0, internal =0.0;
		
		for (int k=0;k<rList.size();k++) {
			if (rList.get(k).isCurrent(fromRequirement)) {
				internal = rList.get(k).getInternal();
			}
		}		
		
		for (int k=0;k<rList.size();k++) {
			if (rList.get(k).isCurrent(toRequirement)) {
				for (int m=0;m<rList.size();m++) {
					if (rList.get(m).isCurrent(fromRequirement)) {
						Requirements fReq = rList.get(m); 
						ripple_indirect = rList.get(k).getIndirectRippleEffects(fReq, fname, internal);
					}
				}
			}
		}
		
		return ripple_indirect;		

	} 
}
